/*
 * FreeRTOS Kernel V10.0.0
 * Copyright (C) 2017 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software. If you wish to use our Amazon
 * FreeRTOS name, please do so in a fair use way that does not cause confusion.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * http://www.FreeRTOS.org
 * http://aws.amazon.com/freertos
 *
 * 1 tab == 4 spaces!
 */

/*-----------------------------------------------------------
 * Simple IO routines to control the LEDs.
 *-----------------------------------------------------------*/

#include "iodefine.h"

#define LED_D1_PIN PORTA.PDR.BIT.B0
#define LED_D2_PIN PORTA.PDR.BIT.B1
#define LED_D3_PIN PORTA.PDR.BIT.B2
#define LED_D4_PIN PORTA.PDR.BIT.B6
#define LED_D1 PORTA.PODR.BIT.B0
#define LED_D2 PORTA.PODR.BIT.B1
#define LED_D3 PORTA.PODR.BIT.B2
#define LED_D4 PORTA.PODR.BIT.B6

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo includes. */
#include "partest.h"

#define partestNUM_LEDS ( 4 )

/*-----------------------------------------------------------*/

void vParTestInitialise( void )
{
  /* Set all LED related pins to output mode. */
  LED_D1_PIN = 1;
  LED_D2_PIN = 1;
  LED_D3_PIN = 1;
  LED_D4_PIN = 1;

  /* Turn off all LEDs. */
  LED_D1 = 0;
  LED_D2 = 0;
  LED_D3 = 0;
  LED_D4 = 0;
}
/*-----------------------------------------------------------*/

void vParTestSetLED( unsigned long ulLED, signed long xValue )
{
  if( ulLED < partestNUM_LEDS )
  {
    if( xValue != 0 )
    {
      /* Turn the LED on. */
      switch( ulLED )
      {
        case 0:  LED_D1 = 1;
            break;
        case 1:  LED_D2 = 1;
            break;
        case 2:  LED_D3 = 1;
            break;
        case 3:  LED_D4 = 1;
            break;
      }
    }
    else
    {
      /* Turn the LED off. */
      switch( ulLED )
      {
        case 0:  LED_D1 = 0;
            break;
        case 1:  LED_D2 = 0;
            break;
        case 2:  LED_D3 = 0;
            break;
        case 3:  LED_D4 = 0;
            break;
      }
    }
  }
}
/*-----------------------------------------------------------*/

void vParTestToggleLED( unsigned long ulLED )
{
  if( ulLED < partestNUM_LEDS )
  {
    switch( ulLED )
    {
      case 0:  LED_D1 ^= 1;
          break;
      case 1:  LED_D2 ^= 1;
          break;
      case 2:  LED_D3 ^= 1;
          break;
      case 3:  LED_D4 ^= 1;
          break;
    }
  }
}
/*-----------------------------------------------------------*/

